#!/usr/bin/env python
# -*- coding: utf-8 -*-


import maya.cmds as cmds




class HmOptionWindow(object):
	'''
	Thiss class implement a basic Maya-Style option window.
	For creating a window with a content just suorce the 2 classes, HmOptionWindow and your Content Class.
	And build the window as shown in the example.

	###############################################################################
	#Window classes initialization
	window = base_win.HmOptionWindow( 'Window_Title', ContentClass() )

	# Execute command
	window.performFromMenu()

	# Show Window
	window.buildAndShowWindow()

	###############################################################################
	 ________________         
	| HmOptionWindow |   
	 ----------------      
	 		|
	 ________________         
	| Hm Options XXX |   
	 ----------------  

	'''
	defauldDim = [600, 300]
	buttonHeight = 26
	btnSpace = 4
	executeTooltip = 'Execute command'

	def __init__(self, window_title, content):
		'''
		Initialize the window.
		
		Args:
		    window_name: The name of the window.

		    content: A class defining the content of the window. MUST contain the following methods:
		    		createUiElements > returning the UI elements of the window
		    		performCommand > the method that performs the command on the click
		    		saveOptionsValues > the method called for saving the UI elements optionsVar
		    		resetOptionsValues > the method called for resetting the UI elements values

		Returns:
		    None

		'''

		if ' ' in window_title:
			window_name = '_'.join(window_title.split(' '))
		else:
			window_name = window_title

		self.window_name = window_name
		self.window_title = window_title
		self.commandName = self.window_title


		# The specific content of the UI
		self.content = content
	



	def buildWindow(self, *Args):
		'''
		This method build all the elements for the window, but still don't show it to the user.
		This because we can get the values from the options whitout the need to display it
		'''

		#Control if already exist
		if cmds.window(self.window_name, exists=True):
			cmds.deleteUI( self.window_name )


		#Create the window
		cmds.window(self.window_name,
					title= self.window_title,
					w = self.defauldDim[0],
					h = self.defauldDim[1],
					menuBar=True )

		#Add the menu bar and the main scroll layout
		self.mainMenu()

		#Add an hidden tab and frame layout	
		#optionBoxTabLayout = cmds.tabLayout( tabsVisible = False, childResizable = True ) #scrollable = True, , 
		form = cmds.formLayout()
		
		mainScroll = cmds.scrollLayout(cr=True) #, bv=True ) # 
		cmds.frameLayout( label = '%s Options' % self.window_title, collapsable= False, mh=10, mw=10 ) #bgs=True,
		
		# print 'CURRENT BEFORE CONTENT PARENT', cmds.setParent( q=True, m=True )
		#Add the content, this procedure now is empty. Will be overrided in the child classes.
		self.content.createUiElements()		
		# print 'CURRENT PARENT', cmds.setParent( q=True, m=True )

		#Add the bottom main buttons
        # we parent them directly to the window, so they will resize right
		self.buttonRow = cmds.rowLayout(adjustableColumn = 1,
                                        rowAttach = [1, 'bottom', 4 ] )
		self.mainButtons()
		
		cmds.formLayout(form,
                        edit=True,
		                attachForm=[(mainScroll, 'top', 15), (mainScroll, 'left', 5),
                                    (mainScroll, 'right', 5),
                                    (self.buttonRow, 'bottom', 5),
                                    (self.buttonRow, 'right', 5),
                                    (self.buttonRow, 'left', 5) ],
		                attachControl=[(mainScroll , 'bottom', 5, self.buttonRow )]
		)
		# ------------------------------------------------------------------------------------------------------------ #

		
	def buildAndShowWindow(self, *Args):
		'''
		This method builds and show the window to the user, used when the options are called
		'''
		self.buildWindow()
		cmds.showWindow( self.window_name )

	def deleteWindow(self, *Args):
		'''
		Pretty easy
		'''
		cmds.deleteUI( self.window_name )


	def mainMenu(self):
		'''
		This implements the main menu items
		'''
		self.menuBarLayout = cmds.menuBarLayout('mainLayout')
		cmds.menu( label='Edit' )
		cmds.menuItem( label='Save settings', c = self.saveOptions )
		cmds.menuItem( label='Reset settings', c = self.resetOptions)
		# cmds.menu( label='Help', helpMenu=True)
		# cmds.menuItem( label='Help on %s' % self.window_name, c = self.openHelp )


	def mainButtons(self):
		'''
		This implements the main buttons arranged in a form layout
		'''
		#Add the main buttons
		buttonForm = cmds.formLayout()
		cmds.button('applyAndCloseBtn', label= self.commandName, recomputeSize = False, height = self.buttonHeight, c = self.performCmd, ann = self.executeTooltip )
		cmds.button('applyBtn', label = 'apply',  recomputeSize = False, height = self.buttonHeight, c = self.applyCmd )
		cmds.button('closeBtn', label = 'cancel',  recomputeSize = False, height = self.buttonHeight, c= self.cancelCmd )






		#Align the buttons
		cmds.formLayout( 
			buttonForm, edit = True, numberOfDivisions = 100,

			attachForm =		[ 	
								('applyAndCloseBtn', 		'top',  	0 ),
								('applyAndCloseBtn', 		'left',  	self.btnSpace ),
								('applyAndCloseBtn', 		'bottom',  	0 ) 
								],
			attachPosition = 	[ 	
								('applyAndCloseBtn', 		'right',  	self.btnSpace,	33 )
								]
			)

		#==============================================================	

		
		cmds.formLayout( 
			buttonForm, edit = True, numberOfDivisions = 100,
			attachForm =		[ 		
								('applyBtn', 				'top',  	0 ),
								('applyBtn', 				'bottom',  	0 )
								],
			attachPosition = 	[ 	
								('applyBtn', 				'left',  	self.btnSpace,		33 ),
							 	('applyBtn', 				'right',  	self.btnSpace,		67 )
							 	]	
			)


		#==============================================================	
		

		cmds.formLayout( 
			buttonForm, edit = True, numberOfDivisions = 100,		 		
			attachForm =		[ 	
								('closeBtn', 				'top',  	0 ),
								('closeBtn', 				'bottom',  	0 ),
								('closeBtn', 				'right',  	self.btnSpace )
								],	
			attachPosition = 	[ 
								('closeBtn', 				'left',  	self.btnSpace,		67 )
							 	]			
			)
		#==============================================================	





	#======================================================================================
	#======================================================================================
	def performFromMenu(self, *Args):
		'''
		This method create an hidden window for reading the options, execute the command and delete the ghost ui
		'''
		#Grate a ghost interface
		self.buildWindow()

		#Execute the command
		self.applyCmd()

		#Delete the ghost window
		self.deleteWindow()


	def applyCmd(self, *Args):
		'''
		Here we call the command, assuning that the interface is opened
		'''
		#Execute the command defined in the content of interface
		self.content.performCommand()


	def cancelCmd(self, *Args):
		'''
		do nothing and close the window
		'''
		self.deleteWindow()


	def performCmd(self, *Args):
		'''
		Perform command and close the window
		'''
		self.applyCmd()
		self.cancelCmd()


	#======================================================================================	
	def saveOptions(self, *Args):
		'''
		Use the common stuff from the base class and the UI method for saving
		'''
		#Save the values
		self.content.saveOptionsValues()

		#Display message
		print 'Options saved for %s' % self.window_name

	def resetOptions(self, *Args):
		#Reset the values
		self.content.resetOptionsValues()

		#Display message 
		print 'Options reset for %s' % self.window_name

		
	def openHelp(self, *Args):
		'''
		Here we will open the help sections related to the passed command
		'''
		print 'Will open the help of %s' % self.window_name
	



